package com.persagy.apm.diagnose.maintenance.service.impl;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.diagnose.config.ComputePropertiesConfig;
import com.persagy.apm.diagnose.config.RabbitConfig;
import com.persagy.apm.diagnose.constant.AlarmConstants;
import com.persagy.apm.diagnose.constant.DiAgnoseConst;
import com.persagy.apm.diagnose.constant.EnumDataTimeType;
import com.persagy.apm.diagnose.indicatorrecord.model.dto.ConfigItemDataSendTimeDTO;
import com.persagy.apm.diagnose.indicatorrecord.model.dto.QueryMonitorIndicatorRecordDTO;
import com.persagy.apm.diagnose.indicatorrecord.service.IMonitorIndicatorRecordService;
import com.persagy.apm.diagnose.maintenance.constant.*;
import com.persagy.apm.diagnose.maintenance.model.dto.AlarmCondition;
import com.persagy.apm.diagnose.maintenance.model.dto.CalculateMethodDTO;
import com.persagy.apm.diagnose.maintenance.model.dto.WeatherDTO;
import com.persagy.apm.diagnose.maintenance.service.IProjectDataRecordService;
import com.persagy.apm.diagnose.service.CenterDataService;
import com.persagy.apm.diagnose.service.EnergyAlarmServiceImpl;
import com.persagy.apm.diagnose.service.SaasService;
import com.persagy.apm.diagnose.service.WeatherServiceImpl;
import com.persagy.apm.diagnose.service.dto.AlarmRuleDTO;
import com.persagy.apm.diagnose.service.dto.ProjectDTO;
import com.persagy.apm.diagnose.service.dto.TimeDataDTO;
import com.persagy.apm.diagnose.timerecord.model.CalculateTimeRecord;
import com.persagy.apm.diagnose.timerecord.service.ICalculateTimeRecordService;
import com.persagy.apm.diagnose.utils.*;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.AddAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.BatchUpdateAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.QueryAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.UpdateAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmRecordItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmRecordServiceImpl;
import com.persagy.framework.ems.data.core.enumeration.SpecialOperator;
import com.persagy.framework.ems.data.mvc.dao.CoreDao;
import com.persagy.framework.ems.data.pojo.hbase.MonitorIndicatorRecord;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * service接口实现 项目数据记录
 */
@Slf4j
@Service
public class ProjectDataRecordServiceImpl implements IProjectDataRecordService {

    @Value(value = "${energy.eq.run.system.code}")
    private String systemCode;

    //测试环境  运行状态 上下限值
    @Value(value = "${energy.eq.run.state.up.limit}")
    private Double runStateUpLimit;
    @Value(value = "${energy.eq.run.state.down.limit}")
    private Double runStateDownLimit;

    //报警分类
    public final static String alarmSplit = "_";
    private final static Long SendDataTimeKeyTime = 24 * 60 * 60 * 1000L;
    private final static Double TwdTemp_Limit = 0.1;

    @Autowired
    private AlarmRecordServiceImpl alarmRecordService;
    @Autowired
    private CenterDataService centerDataService;
    @Autowired
    private ICalculateTimeRecordService calculateTimeRecordService;
    @Autowired
    private IMonitorIndicatorRecordService monitorIndicatorRecordService;
    @Autowired
    private SaasService saasService;
    @Autowired
    private RedisLock lockUtil;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private ComputePropertiesConfig computePropertiesConfig;
    @Autowired
    private EnergyAlarmServiceImpl energyAlarmService;
    @Autowired
    private WeatherServiceImpl weatherService;

    @Resource(name = "ZillionDao")
    private CoreDao coredao;
    @Autowired
    private RabbitConfig rabbitConfig;

    @Override
    @Async("asyncComputeMaintenanceDataExecutor")
    public Future<String> computeMaintenanceData(Date todayDate, ProjectDTO projectDTO, CountDownLatch latch) throws Exception {
        long time = System.currentTimeMillis() + (20 * 1000);
        try {
            if (!lockUtil.lock(projectDTO.getProjectId() + "_computeMaintenance", String.valueOf(time))) {
                // 获取不到锁,略过
                return new AsyncResult<>("【监测维保数据计算线程】项目:" + projectDTO.getProjectId() + "被占用");
            }
            long start = System.currentTimeMillis();
            AsyncResult<String> projectDTOBack = computeProjectMaintenanceData(todayDate, projectDTO);
            if (projectDTOBack != null) {
                return projectDTOBack;
            }
            long end = System.currentTimeMillis();
            return new AsyncResult<>("【监测维保数据计算线程】计算项目指标数据用时:" + projectDTO.getProjectId() + ";" + (end - start));
        } finally {
            // 释放锁
            lockUtil.unlock(projectDTO.getProjectId() + "_computeMaintenance", String.valueOf(time));
            latch.countDown();
        }
    }


    @Override
    @Async("asyncSendMaintenanceDataExecutor")
    public Future<String> sendMaintenanceData(Date todayDate, String projectId, CountDownLatch latch) throws Exception {
        long time = System.currentTimeMillis() + (20 * 1000);
        try {
            if (!lockUtil.lock(projectId + "_sendMaintenance", String.valueOf(time))) {
                // 获取不到锁,略过
                return new AsyncResult<>("【监测维保数据推送线程】:项目:" + projectId + "被占用");
            }
            long start = System.currentTimeMillis();
            sendProjectMaintenanceData(todayDate, projectId);
            long end = System.currentTimeMillis();
            return new AsyncResult<>("【监测维保数据推送线程】推送项目维保数据用时:" + projectId + ";" + (end - start));
        } finally {
            lockUtil.unlock(projectId + "_sendMaintenance", String.valueOf(time));
            latch.countDown();
        }
    }

    @Override
    @Async("asyncGoingDestroyEquipExecutor")
    public Future<String> syncGoingDestroyEquip(String projectId, CountDownLatch latch) {
        long time = System.currentTimeMillis() + (20 * 1000);
        try {
            if (!lockUtil.lock(projectId + "_syncEquip", String.valueOf(time))) {
                // 获取不到锁,略过
                return new AsyncResult<>("【监测同步即将报废设备线程】:项目:" + projectId + "被占用");
            }
            long start = System.currentTimeMillis();
            syncProjectGoingDestroyEquip(projectId);
            long end = System.currentTimeMillis();
            return new AsyncResult<>("【监测同步即将报废设备线程】同步项目即将报废设备用时:" + projectId + ";" + (end - start));
        } finally {
            lockUtil.unlock(projectId + "_syncEquip", String.valueOf(time));
            latch.countDown();
        }
    }

    /**
     * 同步即将报废设备  更新更换设备报警信息
     */
    private void syncProjectGoingDestroyEquip(String projectId) {
        try {
            //获取该项目的更换设备类型的 报警集合
            Map<String, AlarmRecordItem> alarmRecordItemMap = getAlarmRecordItemMap(projectId);
            List<String> alarmEquipList = new ArrayList<>(alarmRecordItemMap.keySet());
            //获取即将报废设备集合
            Map<String, JSONObject> equipInfoMap = getGoingDestroyEquipMap(projectId);
            List<String> gdEquipList = new ArrayList<>(equipInfoMap.keySet());
            alarmEquipList.addAll(gdEquipList);
            Set<String> equipSet = new HashSet<>(alarmEquipList);

            List<String> saveAlarmEquipIdList = new ArrayList<>();
            List<UpdateAlarmRecordDTO> updateAlarmRecordDTOList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(equipSet)) {
                for (String equipId : equipSet) {
                    if (alarmRecordItemMap.containsKey(equipId)) {
                        if (!equipInfoMap.containsKey(equipId)) {
                            //存在于报警记录和但不在报废记录，将报警状态修改为 已恢复
                            AlarmRecordItem alarmRecordItem = alarmRecordItemMap.get(equipId);
                            if (EnumAlarmState.UN_HANDLE.getCode().equals(alarmRecordItem.getState())) {
                                UpdateAlarmRecordDTO updateAlarmRecordDTO = new UpdateAlarmRecordDTO();
                                updateAlarmRecordDTO.setId(alarmRecordItem.getId());
                                updateAlarmRecordDTO.setState(EnumAlarmState.END.getCode());
                                updateAlarmRecordDTO.setEndTime(new Date());

                                updateAlarmRecordDTOList.add(updateAlarmRecordDTO);
                            }
                        }
                        //同时存在于 报废记录和报警记录，跳过不处理
                    } else {
                        //不存在于报警列表  在即将报废列表   新产生报警信息
                        if (equipInfoMap.containsKey(equipId)) {
                            saveAlarmEquipIdList.add(equipId);
                        }
                    }
                }
            }
            //修改报警
            if (!CollectionUtils.isEmpty(updateAlarmRecordDTOList) && updateAlarmRecordDTOList.size() > 0) {
                BatchUpdateAlarmRecordDTO batchUpdateAlarmRecordDTO = new BatchUpdateAlarmRecordDTO();
                batchUpdateAlarmRecordDTO.setAlarmRecordModels(updateAlarmRecordDTOList);
                alarmRecordService.batchUpdate(batchUpdateAlarmRecordDTO);
            }
            //新增报警
            if (!CollectionUtils.isEmpty(saveAlarmEquipIdList) && saveAlarmEquipIdList.size() > 0) {
                List<JSONObject> objectList = centerDataService.queryObjListByObjId(projectId, saveAlarmEquipIdList);
                if (!CollectionUtils.isEmpty(objectList) && objectList.size() > 0) {
                    for (JSONObject object : objectList) {
                        String classCode = object.getString("classCode");//类型编码
                        JSONObject supplement = new JSONObject();
                        supplement.put("specification", object.getString("specification"));//设备型号
                        supplement.put("equip_local_id", object.getString("localId"));//设备本地编码
                        supplement.put("equip_local_name", object.getString("localName"));//设备本地名称
                        //构造报警
                        AddAlarmRecordDTO newItem = initGHAlarmRecordItem(object.getString("id"), projectId, classCode, supplement);
                        alarmRecordService.create(newItem);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 构造 更换设备报警信息
     */
    public AddAlarmRecordDTO initGHAlarmRecordItem(String equipId, String projectId, String classCode, JSONObject supplementObject) {
        AddAlarmRecordDTO alarmRecordItem = new AddAlarmRecordDTO();
        alarmRecordItem.setId(UUID.randomUUID().toString());
        alarmRecordItem.setItemCode(EnumAlarmItem.EquipmentReplace.getCode());//报警条目
        alarmRecordItem.setObjId(equipId);
        alarmRecordItem.setClassCode(classCode);//对象类型编码
        alarmRecordItem.setLevel(EnumAlarmLevel.S.getCode());//严重程度
        alarmRecordItem.setProjectId(projectId);
        alarmRecordItem.setGroupCode(systemCode);
        alarmRecordItem.setRemark("设备使用时间较长，请及时申请更换。");//报警描述
        alarmRecordItem.setName("设备更换");//报警名称
        alarmRecordItem.setCategory(EnumAlarmCategory.EQGH.getCode());//报警分类
        alarmRecordItem.setState(EnumAlarmState.UN_HANDLE.getCode());//报警状态
        alarmRecordItem.setTriggerTime(new Date());//报警触发时间
        alarmRecordItem.setSupplement(supplementObject);//补充说明
        alarmRecordItem.setTreatState(EnumAlarmTreatState.UN_HANDLE.getCode());//处理状态
        //        alarmRecordItem.setEffectStartTime();//有效期开始时间
        //        alarmRecordItem.setEffectEndTime();//有效期结束时间
        //        alarmRecordItem.setNature();//报警性质
        //        alarmRecordItem.setTreatMode();//处理方式
        //        alarmRecordItem.setCondition();//触发和恢复条件
        //        alarmRecordItem.setTriggerInfo();//触发值信息
        //        alarmRecordItem.setEndInfo();//恢复值信息
        //        alarmRecordItem.setEndTime();//恢复时间
        //        alarmRecordItem.setOrderId();//工单id
        //        alarmRecordItem.setOrderState();//工单状态
        //        alarmRecordItem.setIgnoreTime();//忽略时间
        //        alarmRecordItem.setTreatEndTime();//处理时间
        return alarmRecordItem;
    }

    /**
     * 推送维保数据
     */
    private void sendProjectMaintenanceData(Date todayDate, String projectId) throws Exception {
        List<String> itemCodeList = new ArrayList<>();
        itemCodeList.add(EnumAlarmItem.CondAbnormal.getCode());
        itemCodeList.add(EnumAlarmItem.EvapAbnormal.getCode());
        itemCodeList.add(EnumAlarmItem.CoolTowerApproachTempAbnormal.getCode());
        //通过报警服务查询报警规则
        List<AlarmRuleDTO> projectAlarmRules = energyAlarmService.getProjectAlarmRules(projectId, itemCodeList);
        if (CollectionUtils.isEmpty(projectAlarmRules)) {
            return;
        }
        //根据报警规则,获取对象id_报警类型及设备监控点位list
        Map<String, List<String>> objIdAlarmItemCodeAndInfoCodes = getObjIdAlarmItemCodeAndInfoCodes(projectAlarmRules);
        if (CollectionUtils.isEmpty(objIdAlarmItemCodeAndInfoCodes) || objIdAlarmItemCodeAndInfoCodes.size() == 0) {
            return;
        }
        for (Map.Entry<String, List<String>> entry : objIdAlarmItemCodeAndInfoCodes.entrySet()) {
            String objIdAndAlarmItemCode = entry.getKey();
            int indexOf = objIdAndAlarmItemCode.indexOf("_");
            String objId = objIdAndAlarmItemCode.substring(0, indexOf);
            String alarmItemCode = objIdAndAlarmItemCode.substring(indexOf + 1);
            String lockKey = AlarmConstants.alarmDataLockPrefix + objId + "-" + alarmItemCode;
            Object isLock = redisUtil.get(lockKey);
            if (null != isLock) {
                continue;
            }
            long time = System.currentTimeMillis() + (10 * 1000);
            lockUtil.sendDataLock(lockKey, String.valueOf(time));
            List<String> infCodeList = entry.getValue();
            //通过redis查询对象该报警类型下的发数截止时间
            ConfigItemDataSendTimeDTO configItemSendTimeDTO = getConfigItemSendTimeDTO(projectId, objIdAndAlarmItemCode);

            Date lastSendTime = configItemSendTimeDTO.getLastSendTime();
            Date endTime = todayDate;

            List<MonitorIndicatorRecord> list = getMonitorIndicatorRecordsGtLte(projectId, objId, lastSendTime, infCodeList, endTime);
            if (CollectionUtils.isEmpty(list)) {
                continue;
            }
            Map<String, List<MonitorIndicatorRecord>> timeKeyAndRecordList = new LinkedHashMap<>();
            for (MonitorIndicatorRecord record : list) {
                String timeKey = com.persagy.apm.diagnose.utils.DateUtils.date2Str(record.getDataTime(), com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
                if (!timeKeyAndRecordList.containsKey(timeKey)) {
                    timeKeyAndRecordList.put(timeKey, new ArrayList<>());
                }
                timeKeyAndRecordList.get(timeKey).add(record);
            }
            Map<String, List<MonitorIndicatorRecord>> sendTimeKeyAndDataList = new LinkedHashMap<>();
            for (Map.Entry<String, List<MonitorIndicatorRecord>> dateListEntry : timeKeyAndRecordList.entrySet()) {
                List<MonitorIndicatorRecord> value = dateListEntry.getValue();
                if (CollectionUtils.isEmpty(value) || value.size() != infCodeList.size()) {
                    continue;
                }
                sendTimeKeyAndDataList.put(dateListEntry.getKey(), value);
                endTime = com.persagy.apm.diagnose.utils.DateUtils.str2Date(dateListEntry.getKey(), com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
            }
            JSONArray sendArray = CollectDataUtil.batchBuildSendJsonParam(sendTimeKeyAndDataList, alarmItemCode);
            rabbitConfig.sendMsg(sendArray);
            //            String sentValue = CollectDataUtil.batchBuildSendParam(sendTimeKeyAndDataList,alarmItemCode);
            //   AlarmWebSocketServer.sendMsgToClients(projectId, sentValue);
            log.info("设备维保发送数据服务数据:" + projectId + ";" + sendArray);
            configItemSendTimeDTO.setLastSendTime(endTime);
            configItemSendTimeDTO.setUpdateTime(new Date());
            updateRedisConfigItemSendTimeDTO(projectId, objIdAndAlarmItemCode, configItemSendTimeDTO);
        }
    }

    private void updateRedisConfigItemSendTimeDTO(String projectId, String objIdAndAlarmItemCode, ConfigItemDataSendTimeDTO sendTimeDTO) {
        redisUtil.set(getRedisSendDataKey(projectId, objIdAndAlarmItemCode), JSON.toJSONString(sendTimeDTO), SendDataTimeKeyTime);
    }

    private List<MonitorIndicatorRecord> getMonitorIndicatorRecordsGtLte(String projectId, String objId, Date lastSendTime, List<String> infoCodes, Date endTime) throws Exception {
        MonitorIndicatorRecord monitorIndicatorRecord = new MonitorIndicatorRecord();
        monitorIndicatorRecord.setProject(projectId);
        monitorIndicatorRecord.setObjId(objId);
        //        monitorIndicatorRecord.setSplitTimeType(EnumTimeType.MIN15.getCode());
        monitorIndicatorRecord.setSpecialOperation("monitorIndicatorId", SpecialOperator.$in, infoCodes);
        monitorIndicatorRecord.setSpecialOperation("dataTime", SpecialOperator.$gte, lastSendTime);
        monitorIndicatorRecord.setSpecialOperation("dataTime", SpecialOperator.$lt, endTime);

        return coredao.query(monitorIndicatorRecord);
    }

    private Map<String, List<String>> getObjIdAlarmItemCodeAndInfoCodes(List<AlarmRuleDTO> projectAlarmRules) {
        Map<String, List<String>> objIdAlarmItemCodeAndInfoCodes = new HashMap<>();
        for (AlarmRuleDTO projectAlarmRule : projectAlarmRules) {
            String alarmItemCode = projectAlarmRule.getAlarmItemCode();
            List<String> objIds = projectAlarmRule.getObjIds();
            List<String> infoCodes = projectAlarmRule.getInfoCodes();
            if (!CollectionUtils.isEmpty(infoCodes) && !CollectionUtils.isEmpty(objIds)) {
                for (String infoCode : infoCodes) {
                    for (String objId : objIds) {
                        String key = objId + "_" + alarmItemCode;
                        if (!objIdAlarmItemCodeAndInfoCodes.containsKey(key)) {
                            objIdAlarmItemCodeAndInfoCodes.put(key, new ArrayList<>());
                        }
                        objIdAlarmItemCodeAndInfoCodes.get(key).add(infoCode);
                    }
                }
            }
        }
        return objIdAlarmItemCodeAndInfoCodes;
    }

    private ConfigItemDataSendTimeDTO getConfigItemSendTimeDTO(String projectId, String objIdAndAlarmItemCode) throws Exception {
        ConfigItemDataSendTimeDTO sendTimeDTO = null;
        Object obj = redisUtil.get(getRedisSendDataKey(projectId,objIdAndAlarmItemCode));
        if (null != obj){
            sendTimeDTO =  JSONUtil.toBean(obj.toString(), ConfigItemDataSendTimeDTO.class);
        }else{
            sendTimeDTO = ConfigItemDataSendTimeDTO.builder().lastSendTime(getDefultStartTime()).build();
        }
        return sendTimeDTO;
    }

    private Date getDefultStartTime() throws Exception {
        Date lastComputeTime;
        if (org.apache.commons.lang.StringUtils.isBlank(computePropertiesConfig.getDStartTime())) {
            lastComputeTime = new Date();
        } else {
            lastComputeTime = com.persagy.apm.diagnose.utils.DateUtils.str2Date(computePropertiesConfig.getDStartTime(), com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND);
        }
        return lastComputeTime;
    }

    private String getRedisSendDataKey(String projectId, String objIdAndAlarmItemCode) {
        return DiAgnoseConst.RedisConstants.PROJECT_ITEM_SEND_TIME + ":" + projectId + ":" + objIdAndAlarmItemCode;
    }

    /**
     * 计算维保数据
     */
    private AsyncResult<String> computeProjectMaintenanceData(Date todayDate, ProjectDTO projectDTO) {
        try {
            //通过报警服务查询报警规则
            List<String> itemCodeList = new ArrayList<>();
            itemCodeList.add(EnumAlarmItem.CondAbnormal.getCode());
            itemCodeList.add(EnumAlarmItem.EvapAbnormal.getCode());
            itemCodeList.add(EnumAlarmItem.CoolTowerApproachTempAbnormal.getCode());
            List<AlarmRuleDTO> projectAlarmRules = energyAlarmService.getProjectAlarmRules(projectDTO.getProjectId(), itemCodeList);
            if (CollectionUtils.isEmpty(projectAlarmRules)) {
                return new AsyncResult<>("【监测维保数据计算线程】当前项目:" + projectDTO.getProjectId() + "未设置设备维保报警规则");
            }
            Set<String> objIdAndIndicatorId = getObjIdAndInfoCodeFromRules(projectAlarmRules);
            if (CollectionUtils.isEmpty(objIdAndIndicatorId)) {
                return new AsyncResult<>("【监测维保数据计算线程】当前项目:" + projectDTO.getProjectId() + "报警规则未关联对象");
            }
            //Map<objId_monitorIndicatorId, Date>
            Map<String, Date> objId_indicatorIdAndComputeTime = getObjId_IndicatorIdAndComputeTIme(projectDTO.getProjectId());
            //循环计算
            for (AlarmRuleDTO alarmRuleDTO : projectAlarmRules) {
                String itemCode = alarmRuleDTO.getAlarmItemCode();
                EnumAlarmItem enumAlarmItem = EnumAlarmItem.getEnumAlarmLevel(itemCode);
                if (enumAlarmItem == null) {
                    continue;
                }
                List<String> objIdList = alarmRuleDTO.getObjIds();
                List<String> infoCodeList = alarmRuleDTO.getInfoCodes();
                if (CollectionUtils.isEmpty(objIdList) || CollectionUtils.isEmpty(infoCodeList)) {
                    continue;
                }
                for (String objId : objIdList) {
                    List<AlarmCondition> alarmConditionList = energyAlarmService.getProjectAlarmCondition(projectDTO.getProjectId(), objId, enumAlarmItem.getCode());
                    if (CollectionUtils.isEmpty(alarmConditionList)) {
                        continue;
                    }
                    Map<String, CalculateMethodDTO> monitorIndicatorMap = new HashMap<>();
                    AlarmCondition condition = alarmConditionList.get(0);
                    List<List<CalculateMethodDTO>> calculateMethodDTOLists = condition.getTriggerFrontend();
                    if (!CollectionUtils.isEmpty(calculateMethodDTOLists)) {
                        for (List<CalculateMethodDTO> dtoList : calculateMethodDTOLists) {
                            for (CalculateMethodDTO dto : dtoList) {
                                monitorIndicatorMap.put(dto.getMonitorIndicatorId(), dto);
                            }
                        }
                    }

                    for (String infoCode : infoCodeList) {
                        EnumMaintenanceIndicatorType indicatorType = EnumMaintenanceIndicatorType.getEnumMaintenanceDataType(infoCode);
                        if (indicatorType == null) {
                            continue;
                        }
                        //计算持续运行时长跳过
                        if (EnumMaintenanceIndicatorType.ACCCAC_EvapContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCAC_CondContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCCC_EvapContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCCC_CondContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCPC_EvapContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCPC_CondContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCSC_EvapContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCSC_CondContinuousOperation == indicatorType ||
                                EnumMaintenanceIndicatorType.ACCCOT_ContinuousOperation == indicatorType) {
                            continue;
                        }
                        String objIdIndicatorId = objId + alarmSplit + infoCode;
                        //获取上次计算日期
                        Date lastComputeTime = objId_indicatorIdAndComputeTime.get(objIdIndicatorId);
                        if (lastComputeTime == null) {
                            lastComputeTime = DateUtils.addDays(todayDate, -1);
                        }
                        Date computeTime;
                        if (indicatorType == EnumMaintenanceIndicatorType.ACCCOT_DayAvg) {//冷却塔
                            //冷却塔 日均 数据计算
                            computeTime = computeCoolTowerData(lastComputeTime, todayDate, projectDTO, objId, indicatorType);
                        } else {//冷水机组
                            //冷却水组  日均 数据计算
                            computeTime = computeCoolWaterData(lastComputeTime, todayDate, projectDTO.getProjectId(), objId, indicatorType);
                        }
                        //计算 运行持续时长
                        computeAlarmDay(projectDTO.getProjectId(), objId, infoCode, indicatorType, lastComputeTime, todayDate, monitorIndicatorMap);
                        //更新计算 时间记录
                        saveOrUpdateComputeTime(projectDTO.getProjectId(), objId, infoCode, computeTime);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 计算报警持续时长
     */
    public void computeAlarmDay(String projectId, String objId, String indicatorId, EnumMaintenanceIndicatorType indicatorType, Date lastComputeTime, Date todayDate, Map<String, CalculateMethodDTO> configMap) throws Exception {
        //计算 运行持续时长
        CalculateMethodDTO thresholdConfig = null, timeConfig = null;
        EnumMaintenanceIndicatorType saveIndicatorType = null;
        if (null != configMap && configMap.containsKey(indicatorId) && null != configMap.get(indicatorId)) {
            thresholdConfig = configMap.get(indicatorId);
            if (EnumMaintenanceIndicatorType.ACCCAC_CondHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCAC_CondContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCAC_CondContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCAC_CondContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCAC_EvapHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCAC_EvapContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCAC_EvapContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCAC_EvapContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCCC_CondHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCCC_CondContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCCC_CondContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCCC_CondContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCCC_EvapHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCCC_EvapContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCCC_EvapContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCCC_EvapContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCPC_CondHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCPC_CondContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCPC_CondContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCPC_CondContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCPC_EvapHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCPC_EvapContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCPC_EvapContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCPC_EvapContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCSC_CondHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCSC_CondContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCSC_CondContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCSC_CondContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCSC_EvapHeatTransferDayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCSC_EvapContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCSC_EvapContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCSC_EvapContinuousOperation;
            } else if (EnumMaintenanceIndicatorType.ACCCOT_DayAvg == indicatorType) {
                if (configMap.containsKey(indicatorId) && null != configMap.get(EnumMaintenanceIndicatorType.ACCCOT_ContinuousOperation.getCode())) {
                    timeConfig = configMap.get(EnumMaintenanceIndicatorType.ACCCOT_ContinuousOperation.getCode());
                }
                saveIndicatorType = EnumMaintenanceIndicatorType.ACCCOT_ContinuousOperation;
            }
        }
        for (Date start = lastComputeTime; start.before(todayDate); start = DateUtils.addDays(start, 1)) {
            Integer dayNum = 0;
            if (null != thresholdConfig && null != timeConfig && null != saveIndicatorType) {
                //>=、>、<=、<、==
                String operator = thresholdConfig.getOperator();
                String tempFormula = thresholdConfig.getTempFormula();
                if (!StringUtils.isBlank(operator) && !StringUtils.isBlank(tempFormula)) {
                    //获取报警开始时间  不检查当前天的数据
                    Date startAlarmDate = getStartAlarmDate(start, projectId, objId, indicatorId, operator, tempFormula);
                    if (null != startAlarmDate) {
                        dayNum = com.persagy.apm.diagnose.utils.DateUtils.getDayNum(startAlarmDate, start);
                    }
                }
            }
            //构造实体对象
            MonitorIndicatorRecord monitorIndicatorRecord = new MonitorIndicatorRecord();
            monitorIndicatorRecord.setProject(projectId);
            monitorIndicatorRecord.setObjId(objId);
            monitorIndicatorRecord.setMonitorIndicatorId(saveIndicatorType.getCode());
            monitorIndicatorRecord.setDataTime(start);
            monitorIndicatorRecord.setDataValue(Double.valueOf(dayNum));
            monitorIndicatorRecordService.saveOrUpdateByPk(monitorIndicatorRecord);
        }
    }


    /**
     * 获取报警起始日期
     */
    public Date getStartAlarmDate(Date alarmDate, String projectId, String objId, String indicatorId, String operator, String tempFormula) throws Exception {
        // 再次确认检查该日报警  持续天数
        Date checkEndDate =  DateUtils.addDays(alarmDate, 1);
        Date checkStartDate = DateUtils.addDays(alarmDate, -5);
        QueryMonitorIndicatorRecordDTO queryMonitorIndicatorRecordDTO = new QueryMonitorIndicatorRecordDTO();
        queryMonitorIndicatorRecordDTO.setProjectId(projectId);
        queryMonitorIndicatorRecordDTO.setObjId(objId);
        queryMonitorIndicatorRecordDTO.setMonitorIndicatorId(indicatorId);
        queryMonitorIndicatorRecordDTO.setTimeFrom(checkStartDate); // >=
        queryMonitorIndicatorRecordDTO.setTimeTo(checkEndDate);  // <
        List<MonitorIndicatorRecord> recordList = monitorIndicatorRecordService.queryMonitorIndicatorRecordList(queryMonitorIndicatorRecordDTO);
        if (!CollectionUtils.isEmpty(recordList) && recordList.size() > 0) {
            //倒序排
            recordList.sort(Comparator.comparing(MonitorIndicatorRecord::getDataTime).reversed());
            Date checkDate = alarmDate;
            for (MonitorIndicatorRecord record : recordList) {
                boolean isFlag = isAccordWith(operator, Double.valueOf(tempFormula), record.getDataValue());
                if (!isFlag) {
                    return record.getDataTime();
                }
                checkDate = record.getDataTime();
            }
            //继续往前找报警开始日期
            checkDate =  DateUtils.addDays(checkDate, -1);
            return getStartAlarmDate(checkDate, projectId, objId, indicatorId, operator, tempFormula);
        }
        return alarmDate;
    }

    /**
     * 判断是否符合条件
     */
    public boolean isAccordWith(String operator, Double limitVal, Double checkVal) {
        if (!StringUtils.isBlank(operator) && null != limitVal && null != checkVal) {
            if ("≥".equals(operator) && checkVal >= limitVal){
                return true;
            }else if (">".equals(operator) && checkVal > limitVal){
                return true;
            }else if ("≤".equals(operator) && checkVal <= limitVal){
                return true;
            }else if ("<".equals(operator) && checkVal < limitVal){
                return true;
            }else if ("=".equals(operator) && checkVal == limitVal){
                return true;
            }
        }
        return false;
    }

    /**
     * 冷却塔 日均计算
     */
    Date computeCoolTowerData(Date startDate, Date endDate, ProjectDTO projectDTO, String cTObjId, EnumMaintenanceIndicatorType indicatorType) throws Exception {
        //        Date runDate = null;
        //        //获取冷却塔下所有风机运行时时间列表(所有风机运行时间合并为一组时间列表)
        //        Map<Date, List<Date>> allDateRunDataMap = getCoolTowerRunDateMap(projectId, startDate, endDate);
        //        //获取持续运行2小时之后的  时间点集合
        //        Map<Date, List<Date>> dateRunDataMap = getCoolTowerContinuedRunDateMap(allDateRunDataMap);

        Date runDate = null;
        Map<Date, List<TimeDataDTO>> dayTimeDataMap = getServiceDataFromCenterDataService(projectDTO.getProjectId(), cTObjId, EnumCoolWaterPointNameType.RunStatus.getCode(), startDate, endDate);
        //获取日 对应运行时间集合
        Map<Date, List<Date>> dateRunDataMap = getContinuedRunDateMap(dayTimeDataMap);
        //获取冷却水回水干管温度数据集合
        Map<Date, List<TimeDataDTO>> dayCoolWaterMainReturnTempMap = null;
        //获取室外湿球温度数据集合
        Map<Date, List<TimeDataDTO>> dayTwdTempMap = null;
        //计算日均
        if (dateRunDataMap.size() > 0) {
            //获取中央供冷系统对象
            List<JSONObject> object2List = getEqObjectList(projectDTO.getProjectId(), "ACCC");
            if (!CollectionUtils.isEmpty(object2List)) {
                JSONObject object = object2List.get(0);
                String objId = object.getString("id");
                dayCoolWaterMainReturnTempMap = getServiceDataFromCenterDataService(projectDTO.getProjectId(), objId, EnumCoolTowerPointNameType.CoolWaterMainReturnTemp.getCode(), startDate, endDate);
            }
            //计算出 湿球温度数据集合
            dayTwdTempMap = computerWetBallTempMap(projectDTO, startDate, endDate);
            //获取中央供冷系统对象
            //            List<JSONObject> object3List = getEqObjectList(projectDTO.getProjectId(), "WEMMMS");
            //            if (!CollectionUtils.isEmpty(object3List)) {
            //                JSONObject object = object3List.get(0);
            //                String objId = object.getString("id");
            //                dayTwdTempMap = getServiceDataFromCenterDataService(projectDTO.getProjectId(), objId, EnumCoolTowerPointNameType.Twb.getCode(), startDate, endDate);
            //            }
        }
        //循环计算日均
        for (Date dayDate = startDate; dayDate.before(endDate); dayDate = DateUtils.addDays(dayDate, 1)) {
            //计算 日均值 并构造实体对象
            Double dayAvg = computeDayAvg(dayDate, dateRunDataMap, dayCoolWaterMainReturnTempMap, dayTwdTempMap);
            //构造实体对象
            MonitorIndicatorRecord monitorIndicatorRecord = new MonitorIndicatorRecord();
            monitorIndicatorRecord.setMonitorIndicatorId(indicatorType.getCode());
            monitorIndicatorRecord.setDataTime(dayDate);
            monitorIndicatorRecord.setObjId(cTObjId);
            monitorIndicatorRecord.setProject(projectDTO.getProjectId());
            monitorIndicatorRecord.setDataValue(dayAvg);
            monitorIndicatorRecordService.saveDeleterOrUpdateByPk(monitorIndicatorRecord);
            runDate = dayDate;
        }
        return runDate;
    }

    /**
     * 计算冷水机组 日均数据
     */
    Date computeCoolWaterData(Date startDate, Date endDate, String projectId, String objId, EnumMaintenanceIndicatorType indicatorType) throws Exception {
        Date runDate = null;
        Map<Date, List<TimeDataDTO>> dayTimeDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.RunStatus.getCode(), startDate, endDate);
        //获取日 对应运行时间集合
        Map<Date, List<Date>> dateRunDataMap = getContinuedRunDateMap(dayTimeDataMap);
        Map<Date, List<TimeDataDTO>> firstDayTempDataMap = new HashMap<>();
        Map<Date, List<TimeDataDTO>> secondDayTempDataMap = new HashMap<>();
        if (null != dateRunDataMap && dateRunDataMap.size() > 0) {
            //计算换热端差日维度平均值
            if (EnumMaintenanceIndicatorType.ACCCAC_CondHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCCC_CondHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCPC_CondHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCSC_CondHeatTransferDayAvg == indicatorType) {//计算冷凝器
                //获取冷凝温度数据集合
                firstDayTempDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.CondTemp.getCode(), startDate, endDate);
                //获取冷凝器出口水温数据集合
                secondDayTempDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.CondWaterOutTemp.getCode(), startDate, endDate);
            } else if (EnumMaintenanceIndicatorType.ACCCAC_EvapHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCCC_EvapHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCPC_EvapHeatTransferDayAvg == indicatorType ||
                    EnumMaintenanceIndicatorType.ACCCSC_EvapHeatTransferDayAvg == indicatorType) {//计算蒸发器
                //获取蒸发器出口水温数据集合
                firstDayTempDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.EvapWaterOutTemp.getCode(), startDate, endDate);
                //获取蒸发温度数据集合
                secondDayTempDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.EvapTemp.getCode(), startDate, endDate);
            }
        }
        //循环计算日均
        for (Date dayDate = startDate; dayDate.before(endDate); dayDate = DateUtils.addDays(dayDate, 1)) {
            //计算冷凝器/蒸发器 日均值 并构造实体对象
            Double dayAvg = computeDayAvg(dayDate, dateRunDataMap, firstDayTempDataMap, secondDayTempDataMap);
            //构造实体对象
            MonitorIndicatorRecord monitorIndicatorRecord = new MonitorIndicatorRecord();
            monitorIndicatorRecord.setMonitorIndicatorId(indicatorType.getCode());
            monitorIndicatorRecord.setDataTime(dayDate);
            monitorIndicatorRecord.setObjId(objId);
            monitorIndicatorRecord.setProject(projectId);
            monitorIndicatorRecord.setDataValue(dayAvg);
            monitorIndicatorRecordService.saveDeleterOrUpdateByPk(monitorIndicatorRecord);
            runDate = dayDate;
        }
        return runDate;
    }

    /**
     * 计算日均构造实体对象 存储更新数据库表记录
     */
    public Double computeDayAvg(
            Date dayDate, Map<Date, List<Date>> dateRunDataMap, Map<Date, List<TimeDataDTO>> firstDataMap,
            Map<Date, List<TimeDataDTO>> secondDataMap) {
        Double avg = null;
        try {
            if (null != dateRunDataMap && dateRunDataMap.containsKey(dayDate)) {
                List<Date> runDateList = dateRunDataMap.get(dayDate);
                if (!CollectionUtils.isEmpty(runDateList)) {
                    Map<Date, Double> firstDateDataMap = getTimePointDataMap(dayDate, firstDataMap);
                    Map<Date, Double> secondDateDataMap = getTimePointDataMap(dayDate, secondDataMap);
                    //累加计算该日数据
                    Double totalVal = null;
                    int num = 0;
                    for (Date date : runDateList) {
                        //冷凝温度、冷凝器出口水温 均不为空
                        if (firstDateDataMap.containsKey(date) && null != firstDateDataMap.get(date)
                                && secondDateDataMap.containsKey(date) && null != secondDateDataMap.get(date)) {
                            Double divideTemp = DataUtils.subtract(firstDateDataMap.get(date), secondDateDataMap.get(date));
                            totalVal = totalVal == null ? divideTemp : DataUtils.plus(totalVal, divideTemp);
                            num++;
                        }
                    }
                    //计算均值
                    if (null != totalVal && 0 != num) {
                        avg = totalVal / num;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return avg;
    }

    /**
     * 构造时间点对应数据值集合
     */
    public Map<Date, Double> getTimePointDataMap(Date dayDate, Map<Date, List<TimeDataDTO>> dayDataListMap) {
        Map<Date, Double> dateDataMap = new HashMap<>();
        try {
            if (null != dayDataListMap && dayDataListMap.containsKey(dayDate)) {
                List<TimeDataDTO> dataDTOList = dayDataListMap.get(dayDate);
                if (!CollectionUtils.isEmpty(dataDTOList)) {
                    for (TimeDataDTO dto : dataDTOList) {
                        if (null != dto.getData()) {
                            dateDataMap.put(com.persagy.apm.diagnose.utils.DateUtils.str2Date(dto.getTime(), com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND), dto.getData());
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateDataMap;
    }

    /**
     * 获取日  运行状态时间点集合
     */
    public Map<Date, List<Date>> getContinuedRunDateMap(Map<Date, List<TimeDataDTO>> dayTimeDataMap) {
        Map<Date, List<Date>> dateRunDataMap = new HashMap<>();
        try {
            if (null != dayTimeDataMap && dayTimeDataMap.size() > 0) {
                for (Date dayDate : dayTimeDataMap.keySet()) {
                    List<TimeDataDTO> dataDTOList = dayTimeDataMap.get(dayDate);
                    if (CollectionUtils.isEmpty(dataDTOList)) {
                        continue;//该日数据为空
                    }
                    //正序排
                    dataDTOList.sort(Comparator.comparing(TimeDataDTO::getTime));
                    String firstTime = getFirstRunDate(dataDTOList);
                    if (StringUtils.isBlank(firstTime)) {
                        continue;
                    }
                    //
                    Date firstDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(firstTime, com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
                    Date twoHourDate = DateUtils.addHours(firstDate, 2);
                    boolean isTwoHourRun = checkTwoHourRun(firstDate, twoHourDate, dataDTOList);
                    if (!isTwoHourRun) {
                        continue;
                    }
                    List<Date> dateList = new ArrayList<>();
                    for (TimeDataDTO dto : dataDTOList) {
                        if (null == dto.getData()) {
                            continue;
                        }
                        String time = dto.getTime();
                        Date getDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(time, com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
                        if (getDate.compareTo(twoHourDate) > 0) {
                            //是否为运行状态
                            Boolean isRun = getRunStatueFlag(true,dto.getData());
                            if (isRun){
                                dateList.add(getDate);
                            }
                        }
                    }
                    dateRunDataMap.put(dayDate, dateList);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateRunDataMap;
    }

    /**
     * 获取冷却塔风机 持续运行两小时后  运行状态下的时间点集合
     */
    public Map<Date, List<Date>> getCoolTowerContinuedRunDateMap(Map<Date, List<Date>> dateRunDataMap) {
        if (null != dateRunDataMap && dateRunDataMap.size() > 0) {
            for (Date dayDate : dateRunDataMap.keySet()) {
                List<Date> allDateList = dateRunDataMap.get(dayDate);
                if (CollectionUtils.isEmpty(allDateList)) {
                    continue;
                }
                //正序排
                Collections.sort(allDateList);
                Date firstDate = allDateList.get(0);
                Date twoHourDate = DateUtils.addHours(firstDate, 2);
                List<Date> dateList = new ArrayList<>();
                for (Date date : allDateList) {
                    if (date.compareTo(twoHourDate) > 0) {
                        dateList.add(date);
                    }
                }
                dateRunDataMap.put(dayDate, dateList);
            }
        }
        return dateRunDataMap;
    }


    /**
     * 获取日  运行状态时间点集合
     */
    public Map<Date, List<Date>> getCoolTowerRunDateMap(String projectId, Date startDate, Date endDate) {
        Map<Date, List<Date>> dateRunDataMap = new HashMap<>();
        try {
            //获取所有风机 运行状态数据集合
            Map<Date, List<TimeDataDTO>> dayTimeDataMap = new HashMap<>();
            //获取冷却塔下风机对象列表
            List<JSONObject> objectList = getEqObjectList(projectId, "ACCCOTFN");
            if (!CollectionUtils.isEmpty(objectList) && objectList.size() > 0) {
                for (JSONObject obj : objectList) {
                    String objId = obj.getString("id");
                    Map<Date, List<TimeDataDTO>> objDayTimeDataMap = getServiceDataFromCenterDataService(projectId, objId, EnumCoolWaterPointNameType.RunStatus.getCode(), startDate, endDate);
                    dayTimeDataMap.putAll(objDayTimeDataMap);
                }
            }
            //合并出日对应运行状态时间列表
            if (dayTimeDataMap.size() > 0) {
                for (Date dayDate : dayTimeDataMap.keySet()) {
                    List<TimeDataDTO> dataDTOList = dayTimeDataMap.get(dayDate);
                    if (CollectionUtils.isEmpty(dataDTOList)) {
                        continue;//该日数据为空
                    }
                    List<Date> dateList = new ArrayList<>();
                    if (dateRunDataMap.containsKey(dayDate)) {
                        dateList = dateRunDataMap.get(dayDate);
                    }
                    for (TimeDataDTO dto : dataDTOList) {
                        if (null == dto.getData()) {
                            continue;
                        }
                        Date getDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(dto.getTime(), com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
                        //是否在运行状态
                        Boolean isRun = getRunStatueFlag(true,dto.getData());
                        if (isRun){
                            dateList.add(getDate);
                        }
                    }
                    Set<Date> dateSet = new HashSet<>(dateList);
                    dateList.clear();
                    dateList.addAll(dateSet);
                    dateRunDataMap.put(dayDate, dateList);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateRunDataMap;
    }

    /**
     * 检查是否自开机持续运行2小时
     */
    public boolean checkTwoHourRun(Date firstDate, Date twoHourDate, List<TimeDataDTO> dataDTOList) {
        try {
            for (TimeDataDTO dto : dataDTOList) {
                String time = dto.getTime();
                Date getDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(time, com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
                if (getDate.compareTo(firstDate) >= 0 && getDate.compareTo(twoHourDate) <= 0) {
                    if (null == dto.getData()) {
                        continue;
                    }
                    //判断是否为关闭
                    Boolean isRun = getRunStatueFlag(false,dto.getData());
                    if (!isRun){
                        return false;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    /**
     * 获取开机时间
     */
    public String getFirstRunDate(List<TimeDataDTO> dataDTOList) {
        try {
            for (TimeDataDTO dto : dataDTOList) {
                if (null == dto.getData()) {
                    continue;
                }
                Boolean isRun = getRunStatueFlag(true,dto.getData());
                if (isRun){
                    return dto.getTime();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return null;
    }

    /**
     * 获取数据
     */
    private Map<Date, List<TimeDataDTO>> getServiceDataFromCenterDataService(
            String projectId, String objId, String computeCode,
            Date startTime, Date endTime) throws Exception {
        List<TimeDataDTO> dataList = centerDataService.queryObjTimeDataList(projectId, objId, computeCode, startTime, endTime, EnumDataTimeType.MIN15);
        if (CollectionUtils.isEmpty(dataList)) {
            log.error("【维保计算线程】:查询数据中台对象历史数据为空,projectId:" + projectId + ";objId:" + objId + ";信息点:" + computeCode
                    + ";时间:" + com.persagy.apm.diagnose.utils.DateUtils.date2Str(startTime, com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND) + "_"
                    + com.persagy.apm.diagnose.utils.DateUtils.date2Str(endTime, com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND));
        }
        Map<Date, List<TimeDataDTO>> dayTimeDataMap = new HashMap<>();
        for (TimeDataDTO dto : dataList) {
            String time = dto.getTime();
            Date timeDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(time, com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND);
            Date dayDate = DateUtils.truncate(timeDate, Calendar.DATE);
            if (dayTimeDataMap.containsKey(dayDate)) {
                dayTimeDataMap.get(dayDate).add(dto);
            } else {
                List<TimeDataDTO> dataDTOS = new ArrayList<>();
                dataDTOS.add(dto);
                dayTimeDataMap.put(dayDate, dataDTOS);
            }
        }
        return dayTimeDataMap;
    }

    /**
     * 获取设备对象
     */
    public List<JSONObject> getEqObjectList(String projectId, String classCode) {
        List<JSONObject> jsonToList = new ArrayList<>();
        try {
            List<String> classCodeList = new ArrayList<>();
            classCodeList.add(classCode);
            JSONArray jsonArray = centerDataService.queryObjListByObjType(projectId, classCodeList);
            jsonToList = JSON.parseArray(jsonArray.toJSONString(), JSONObject.class);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return jsonToList;
    }

    /**
     * 获取报警记录
     */
    private Map<String, AlarmRecordItem> getAlarmRecordItemMap(String projectId) throws Exception {
        List<String> cateGoryList = new ArrayList<>();
        cateGoryList.add(EnumAlarmCategory.EQGH.getCode());
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setGroupCode(systemCode);
        queryAlarmRecordDTO.setProjectId(projectId);
        queryAlarmRecordDTO.setCategory(cateGoryList);
        DmpResult<List<AlarmRecordItem>> configItemResultList = alarmRecordService.query(queryAlarmRecordDTO);
        List<AlarmRecordItem> alarmRecordItemList = DmpResultUtil.getData(configItemResultList);
        if (!CollectionUtils.isEmpty(alarmRecordItemList)) {
            return alarmRecordItemList.stream().collect(Collectors.toMap(AlarmRecordItem::getObjId, Function.identity(), (a, b) -> b));
        }
        return new HashMap<>();
    }


    private Map<String, Date> getObjId_IndicatorIdAndComputeTIme(String projectId) {
        List<CalculateTimeRecord> calculateTimeRecords = calculateTimeRecordService.queryTimeRecordListByProjectId(projectId);
        return calculateTimeRecords.stream().collect(Collectors.toMap(a -> a.getObjId() + alarmSplit + a.getMonitorIndicatorId(), CalculateTimeRecord::getCalculateLastTime));
    }

    /**
     * 获取即将报废设备集合
     */
    private Map<String, JSONObject> getGoingDestroyEquipMap(String projectId) {
        Map<String, JSONObject> equipInfoMap = new HashMap<>();
        try {
            JSONArray jsonArray = saasService.getGoingDestroyEquipList(projectId);
            if (!CollectionUtils.isEmpty(jsonArray)) {
                for (Object object : jsonArray) {
                    JSONObject obj = (JSONObject) object;
                    equipInfoMap.put(obj.getString("equip_id"), obj);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return equipInfoMap;
    }

    /**
     * 更新缓存中报警配置发数时间
     */
    //    private void updateRedisConfigItemSendTimeDTO(String projectId, ConfigItemDataSendTimeDTO sendTimeDTO) {
    //        redisUtil.hset(RedisUtil.PROJECT_ITEM_SEND_TIME + ":" + projectId, sendTimeDTO.getConfigItemid(), sendTimeDTO, SendDataTimeKeyTime);
    //    }
    private void saveOrUpdateComputeTime(String projectId, String objId, String indicatorId, Date computeTime) {
        if (computeTime == null) {
            return;
        }
        CalculateTimeRecord update = new CalculateTimeRecord();
        update.setMonitorIndicatorId(indicatorId);
        update.setObjId(objId);
        update.setProjectId(projectId);
        update.setCalculateLastTime(computeTime);
        calculateTimeRecordService.saveOrUpdateByPk(update);
    }

    /**
     * 通过报警条件获取所有对象及其监控点位,objId_infoCode
     */
    private Set<String> getObjIdAndInfoCodeFromRules(List<AlarmRuleDTO> projectAlarmRules) {
        Set<String> objIdAndInfoCode = new HashSet<>();
        for (AlarmRuleDTO projectAlarmRule : projectAlarmRules) {
            List<String> objIds = projectAlarmRule.getObjIds();
            if (CollectionUtils.isEmpty(objIds)) {
                continue;
            }
            List<String> infoCodes = projectAlarmRule.getInfoCodes();
            if (CollectionUtils.isEmpty(infoCodes)) {
                continue;
            }
            for (String infoCode : infoCodes) {
                for (String objId : objIds) {
                    objIdAndInfoCode.add(objId + "_" + infoCode);
                }
            }
        }
        return objIdAndInfoCode;
    }

    public Map<Date, List<TimeDataDTO>> computerWetBallTempMap(ProjectDTO projectDTO, Date startDate, Date endDate) {
        try {
            //根据室外天气数据计算出湿球温度数据
            Map<Date, List<WeatherDTO>> dayWeatherDataMap = getWeatherTemp(projectDTO, startDate, endDate);
            if (null != dayWeatherDataMap && dayWeatherDataMap.size() > 0) {
                Map<Date, List<TimeDataDTO>> dayTwdTempMap = new HashMap<>();
                for (Date dayDate : dayWeatherDataMap.keySet()) {
                    List<WeatherDTO> dtoList = dayWeatherDataMap.get(dayDate);
                    if (CollectionUtils.isEmpty(dtoList)) {
                        continue;
                    }
                    List<TimeDataDTO> timeDataDTOList = new ArrayList<>();
                    for (WeatherDTO dto : dtoList) {
                        //根据温度和湿度 计算出湿空气焓值
                        Double hVal = getHVal(dto.getTemperature(), dto.getHumidity());
                        //根据允许范围推出Tw值
                        Double Tw = getTw(dto.getTemperature(), hVal, TwdTemp_Limit);
//                        log.info("项目编号为:" + projectDTO.getProjectId() + "日期为:" + dto.getHourTime() + "温度：" + dto.getTemperature() + "湿度：" + dto.getHumidity() + "湿球温度：" + Tw);
                        //小时时间
                        Date hour = dto.getHourTime();
                        Date nextHour = com.persagy.apm.diagnose.utils.DateUtils.addHours(hour, 1);
                        for (Date startHour = hour; startHour.before(nextHour); startHour = com.persagy.apm.diagnose.utils.DateUtils.addMinutes(startHour, 15)) {
                            TimeDataDTO timeDataDTO = new TimeDataDTO();
                            timeDataDTO.setTime(com.persagy.apm.diagnose.utils.DateUtils.date2Str(startHour, com.persagy.apm.diagnose.utils.DateUtils.SDFSECOND));
                            timeDataDTO.setData(Tw);
                            timeDataDTOList.add(timeDataDTO);
                        }
                    }
                    dayTwdTempMap.put(dayDate, timeDataDTOList);
                }
                return dayTwdTempMap;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取项目 天气数据
     */
    public Map<Date, List<WeatherDTO>> getWeatherTemp(ProjectDTO projectDTO, Date startDate, Date endDate) {
        try {
            String startTime = com.persagy.apm.diagnose.utils.DateUtils.date2Str(startDate, com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND);
            String endTime = com.persagy.apm.diagnose.utils.DateUtils.date2Str(endDate, com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND);
            List<WeatherDTO> weatherDTOList = weatherService.hourHistoryData(projectDTO.getCity(), startTime, endTime, true, 2);
            if (!CollectionUtils.isEmpty(weatherDTOList)) {
                Map<Date, List<WeatherDTO>> dayTimeWeatherDataMap = new HashMap<>();
                for (WeatherDTO dto : weatherDTOList) {
                    //                    String time = dto.getHourTime();
                    //                    Date timeDate = com.persagy.apm.diagnose.utils.DateUtils.str2Date(time, com.persagy.apm.diagnose.utils.DateUtils.SDF_SECOND);
                    Date dayDate = DateUtils.truncate(dto.getHourTime(), Calendar.DATE);
                    if (dayTimeWeatherDataMap.containsKey(dayDate)) {
                        dayTimeWeatherDataMap.get(dayDate).add(dto);
                    } else {
                        List<WeatherDTO> dataDTOS = new ArrayList<>();
                        dataDTOS.add(dto);
                        dayTimeWeatherDataMap.put(dayDate, dataDTOS);
                    }
                }
                return dayTimeWeatherDataMap;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据允许范围推出Tw值
     */
    public Double getTw(Double Td, Double hVal, Double limit) throws InterruptedException {
        Double Tw = Td - 3;
        Double num = 0.0;
        Boolean isOk = false;
        Boolean isTrueVal = false;
        Integer zNum = 0, jNum = 0;
        while (!isOk) {
            //湿空气焓值
            Double hwVal = getHwVal(Tw);
            //两种湿空气焓值  差值
            Double hc = DataUtils.subtract(hVal, hwVal);
            //判断范围
            if (hc < limit && hc > 0.0) {
                isOk = true;
                isTrueVal = true;
            } else if (hc > 0) {//hw 小于 h  增加湿球温度数据值
                Tw = Tw + 0.25;
                zNum++;
            } else if (hc < 0) {//hw 大于 h  减小湿球温度数据值
                Tw = Tw - 0.25;
                jNum++;
            }
            num++;
            if (num > 10) {
                isOk = true;
            }
        }
        if (isTrueVal) {
            return Tw;
        } else {
            if (zNum > 0 && jNum > 0) {
                return Tw;
            } else {
                return getTw(Tw, hVal, limit + 0.5);
            }
        }
    }

    /**
     * 根据湿球温度计算出湿空气焓值
     */
    public double getHwVal(Double Tw) {
        Double eVal = (18.678 - Tw / 234.5) * Tw / (Tw + 257.14);
        Double ev = Math.exp(eVal);
        Double Pv = 611.2 * ev; //Td时的水蒸汽饱和压力
        //含湿量
        Double dw = 0.6219 * (Pv / (101326 - Pv));
        //湿空气焓值
        Double hw = 1.01 * Tw + (2500 + 1.84 * Tw) * dw;
        return hw;
    }

    /**
     * 根据温度和湿度 计算出湿空气焓值
     */
    public double getHVal(Double Td, Double Hr) {
        Double eVal = (18.678 - Td / 234.5) * Td / (Td + 257.14);
        Double ev = Math.exp(eVal);
        Double Pv = 611.2 * ev; //Td时的水蒸汽饱和压力
        Double hrPv = 0.01 * Hr * Pv;
        //含湿量
        Double d = 0.6219 * (hrPv / (101326 - hrPv));
        //湿空气焓值
        Double h = 1.01 * Td + (2500 + 1.84 * Td) * d;
        return h;
    }


    /**
     * 判断是否为运行状态  或者停止状态
     */
    public Boolean getRunStatueFlag(Boolean openOrClose,Double data){
        //获取运行状态时间
        if (openOrClose){
            //正式环境  运行状态采用0/1 进行判断
            if (Objects.equals(EnumRunStatusType.Open.getCode(), data)) {
                return true;
            }
            //测试环境 无法造出0/1 故采用区间数据来判断 运行状态
//            if (data > runStateDownLimit && data < runStateUpLimit) {
//                return true;
//            }
            return false;
        }
        //获取是否在关闭状态
        if (!openOrClose){
//            测试 运行状态
//            if (data < runStateDownLimit || data > runStateUpLimit) {
//                return false;
//            }
            if (Objects.equals(EnumRunStatusType.Close.getCode(), data)) {
                return false;
            }
            return true;
        }
        return false;
    }


}
